Q <- commandArgs(TRUE)[1]
if (is.na(Q)) Q <- "small"

BnS <- function(script, fname, queue=Q) {
  f <- file(fname, 'w')
  writeLines(c("#!/bin/bash",
               paste("#PBS -N", fname),
               "#PBS -S /bin/bash",
               "#PBS -j oe",
               "#PBS -d .",
               "#PBS -l walltime=84:00:00",
               "#PBS -l mem=1gb",
               "#PBS -l nodes=1:ppn=1",
               "module load R; module load gcc;",
               script), f)
  close(f)
  if (queue == 'none') {
    system(paste('cat', fname))
  } else if (queue == 'local') {
    system(script)
  } else {
    system(paste('qsub', fname))
    Sys.sleep(1)
  }
  return(0)
}

token_dynamics <- list(steady_1  = c(gamma=0.25, sigma=400),
                       cycles_1  = c(gamma=0.75, sigma=400),
                       steady_2  = c(gamma=0.90, sigma=400),
                       exclusion = c(gamma=0.99, sigma=400))

for (dynamics in token_dynamics) {
  for (rseed in 0:9) {
    BnS(paste("Rscript MANTIS_state_space_replicability.R",
              rseed, dynamics["gamma"], dynamics["sigma"]),
        paste0(paste("SSR", rseed, dynamics["gamma"], dynamics["sigma"], sep="_"), ".sh"),
        Q)
  }
}
