library(magrittr)
library(tidyverse)
library(broom)
# library(randomForest)
library(kableExtra)

load("~/Desktop/tmp22.RData")

## usually sink has same dynamics as source
mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ "extinct",
                              V3 == 1 ~ "steady",
                              # V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
                              V3 > 1  ~ "cycles")) %>%
  select(-V3) %>%
  group_by(V1) %>%
  do(bind_cols(filter(., V2 == 1) %>% transmute(gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
               filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))) %>%
  ungroup() %>%
  select(dynamics_1, dynamics_2) %>%
  table() %>%
  divide_by(rowSums(.)) %>%
  multiply_by(100) %>%
  {ifelse(is_greater_than(., 0.01), round(., 2), .)} %>%
  format(digits=2, scientific=FALSE, drop0trailing=TRUE) %>%
  apply(1:2, str_c, "%") %>%
  as_tibble(rownames = "Source \\ Sink") %>%
  kable(format="latex", align=str_split("lrrr", "", simplify=TRUE), booktabs=TRUE)

mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ "extinct",
                              V3 == 1 ~ "steady",
                              # V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
                              V3 > 1  ~ "cycles")) %>%
  select(-V3) %>%
  group_by(V1) %>%
  do(bind_cols(filter(., V2 == 1) %>% transmute(gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
               filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))) %>%
  ungroup() %>%
  filter(dynamics_1 == "cycles", dynamics_2 == "steady")


mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  {bind_cols(filter(., V2 == 1) %>% transmute(strain=V1, gamma_1=V4, sigma_1=V5, dynamics_1=V3),
             filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=V3))} %>%
  filter(dynamics_1 > 1 | dynamics_2 > 1) %>%
  {ggplot(.) +
      aes(x=dynamics_1, y=dynamics_2) +
      # stat_density2d(aes(fill=log(stat(density))), geom="raster", contour=FALSE, interpolate=TRUE) +
      geom_hex(aes(fill=log(..density..), colour=log(..density..)), bins=75) +
      # geom_point(alpha=0.25) +
      geom_abline(size=1) +
      scale_colour_viridis_c(option="B", aesthetics = c("fill", "colour"),
                             breaks=c(0, 0.5, 1)) +
      # breaks=c(-10, -7.5, -5), labels=c("Low", "------>", "High")) +
      coord_equal(xlim=c(-0.01, 1.01) * max(.$dynamics_1),
                  ylim=c(-0.01, 1.01) * max(.$dynamics_2),
                  expand=FALSE) +
      xlab("Source Population Dynamics\n(number of local extrema in final 5% of timesteps)") +
      ylab("Sink Population Dynamics\n(number of local extrema in final 5% of timesteps)") +
      theme_bw() +
      theme(legend.position=c(0.2, 0.9),
            legend.background=element_blank()) +
      guides(colour=guide_colourbar(title.position="top", title.hjust = 0.5,
                                    ticks=FALSE, direction="horizontal"))}
ggsave("../Figures/source_sink_cycle_complexity.pdf", width=5, height=5)

mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ "extinct",
                              V3 == 1 ~ "steady",
                              # V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
                              V3 > 1  ~ "cycles")) %>%
  select(-V3) %>%
  {bind_cols(filter(., V2 == 1) %>% transmute(strain=V1, gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
             filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))} %>%
  group_by(gamma_1, sigma_1, gamma_2, sigma_2) %>%
  summarise(dyn_1 = str_c(dynamics_1, collapse=""),
            dyn_2 = str_c(dynamics_2, collapse=""),
            n_dyn_1 = length(unique(dynamics_1)),
            n_dyn_2 = length(unique(dynamics_2))) %>%
  filter(dyn_1 != dyn_2)

mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ "extinct",
                              V3 == 1 ~ "steady",
                              V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
                              V3 > 1  ~ "cycles")) %>%
  select(-V3) %>%
  {bind_cols(filter(., V2 == 1) %>% transmute(strain=V1, gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
             filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))} %>%
  filter(dynamics_1 != dynamics_2, dynamics_1 != "extinct")


## reduced transference when either sigma is high or if gamma2 is high
mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ "extinct",
                              V3 == 1 ~ "steady",
                              V3 > 1  ~ "cycles")) %>%
  select(-V3) %>%
  {bind_cols(filter(., V2 == 1) %>% transmute(strain=V1, gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
             filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))} %>%
  group_by(gamma_1, sigma_1, gamma_2, sigma_2) %>%
  summarise(dyn_1 = str_c(dynamics_1, collapse=""),
            dyn_2 = str_c(dynamics_2, collapse=""),
            n_dyn_1 = length(unique(dynamics_1)),
            n_dyn_2 = length(unique(dynamics_2))) %>%
            {bind_rows(group_by(., gamma_1, sigma_1) %>%
                         summarise(prop = sum(dyn_1 == dyn_2) / n()) %>%
                         rename(gamma=gamma_1, sigma=sigma_1) %>% ungroup() %>%
                         mutate(population = "x*' = '*gamma[' i,1']*'     '*y*' = '*sigma[' i,1']",
                                x=as.integer(as.factor(gamma)),
                                y=as.integer(as.factor(sigma))),
                       group_by(., gamma_2, sigma_2) %>%
                         summarise(prop = sum(dyn_1 == dyn_2) / n()) %>%
                         rename(gamma=gamma_2, sigma=sigma_2) %>% ungroup() %>%
                         mutate(population = "x*' = '*gamma[' i,2']*'     '*y*' = '*sigma[' i ,2']",
                                x=as.integer(as.factor(gamma)),
                                y=as.integer(as.factor(sigma))),
                       group_by(., gamma_1, sigma_2) %>%
                         summarise(prop = sum(dyn_1 == dyn_2) / n()) %>%
                         rename(gamma=gamma_1, sigma=sigma_2) %>% ungroup() %>%
                         mutate(population = "x*' = '*gamma[' i,1']*'     '*y*' = '*sigma[' i, 2']",
                                x=as.integer(as.factor(gamma)),
                                y=as.integer(as.factor(sigma))),
                       group_by(., gamma_2, sigma_1) %>%
                         summarise(prop = sum(dyn_1 == dyn_2) / n()) %>%
                         rename(gamma=gamma_2, sigma=sigma_1) %>% ungroup() %>%
                         mutate(population = "x*' = '*gamma[' i,2']*'     '*y*' = '*sigma[' i,1']",
                                x=as.integer(as.factor(gamma)),
                                y=as.integer(as.factor(sigma))),
                       group_by(., gamma_1, gamma_2) %>%
                         summarise(prop = sum(dyn_1 == dyn_2) / n()) %>%
                         rename(gamma=gamma_1, sigma=gamma_2) %>% ungroup() %>%
                         mutate(population = "x*' = '*gamma[' i,1']*'     '*y*' = '*gamma[' i,2']",
                                x=as.integer(as.factor(gamma)),
                                y=as.integer(as.factor(sigma))),
                       group_by(., sigma_1, sigma_2) %>%
                         summarise(prop = sum(dyn_1 == dyn_2) / n()) %>%
                         rename(gamma=sigma_1, sigma=sigma_2) %>% ungroup() %>%
                         mutate(population = "x*' = '*sigma[' i,1']*'     '*y*' = '*sigma['i,2']",
                                x=as.integer(as.factor(gamma)),
                                y=as.integer(as.factor(sigma))))} %>%
                                {ggplot(.) +
                                    aes(x=x, y=y, fill=prop) +
                                    geom_raster(interpolate=TRUE) +
                                    coord_cartesian(expand=FALSE) +
                                    facet_wrap(~population, labeller=label_parsed) +
                                    scale_fill_viridis_c(name="proportion of cases where the sink dynamics match those of the source", limits=c(0,1), option="D") +
                                    scale_x_continuous(breaks=seq(1,10,3)+0.5, labels=c("Low", rep("", 2), "High")) +
                                    scale_y_continuous(breaks=seq(1,10,3)+0.5, labels=c("Low", rep("", 2), "High")) +
                                    theme(legend.position="bottom") +
                                    guides(fill=guide_colourbar(title.position="top", title.hjust = 0.5,
                                                                barwidth=unit(500, "pt")))}
# ggsave("../Figures/liklihood_of_dynamics_transference.pdf", width=10, height=8)

## TOO MEMORY INTENSIVE
# mantis_output %>%
#   as.data.frame() %>%
#   mutate_all(unlist) %>%
#   as_tibble() %>%
#   mutate(dynamics = case_when(V3 == 0 ~ "extinct",
#                               V3 == 1 ~ "steady",
#                               # V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
#                               V3 > 1  ~ "cycles")) %>%
#   select(-V3) %>%
#   {bind_cols(filter(., V2 == 1) %>% transmute(strain=V1, gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
#              filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))} %>%
#   mutate(dynamics_1 = as.factor(dynamics_1), dynamics_2 = as.factor(dynamics_2)) %>%
#   randomForest(dynamics_2 ~ dynamics_1 +  gamma_1 * sigma_1 + gamma_2 + sigma_2, data=.) %>%
#   tidy()

## almost always goes to more complex dynamics (when it changes, that is)
mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ 1,
                              V3 == 1 ~ 2,
                              # V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
                              V3 > 1  ~ 3)) %>%
  select(-V3) %>%
  {bind_cols(filter(., V2 == 1) %>% transmute(strain=V1, gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
             filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))} %>%
  group_by(gamma_1, sigma_1, gamma_2, sigma_2) %>%
  summarise(dyn_1 = dynamics_1[2],
            dyn_2 = dynamics_2[2],
            n_dyn_1 = length(unique(dynamics_1)),
            n_dyn_2 = length(unique(dynamics_2))) %>%
  filter(dyn_1 > dyn_2) %>% as.data.frame()


mantis_output %>%
  as.data.frame() %>%
  mutate_all(unlist) %>%
  as_tibble() %>%
  mutate(dynamics = case_when(V3 == 0 ~ "extinct",
                              V3 == 1 ~ "steady",
                              # V3 > quantile(V3[V3 > 1], 0.95)  ~ "chaos",
                              V3 > 1  ~ "cycles")) %>%
  select(-V3) %>%
  group_by(V1) %>%
  do(bind_cols(filter(., V2 == 1) %>% transmute(gamma_1=V4, sigma_1=V5, dynamics_1=dynamics),
               filter(., V2 == 2) %>% transmute(gamma_2=V4, sigma_2=V5, dynamics_2=dynamics))) %>%
  ungroup() %>%
  select(-V1) %>%
  gather("variable", "value", -dynamics_1, -dynamics_2) %>%
  {ggplot(.) + aes(x=value, y=..density.., fill=(dynamics_1 != dynamics_2 & dynamics_1 != "extinct")) + geom_histogram(alpha=0.5, position="identity", bins=20) + facet_wrap(~variable, scales="free") + theme(legend.position="bottom")}
